package com.bundle.bundle_rave_pay;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;

import com.bundle.bundle_rave_pay.account.AccountFragment;
import com.bundle.bundle_rave_pay.banktransfer.BankTransferFragment;
import com.bundle.bundle_rave_pay.card.CardFragment;
import com.bundle.bundle_rave_pay.di.components.AppComponent;
import com.bundle.bundle_rave_pay.di.components.DaggerAppComponent;
import com.bundle.bundle_rave_pay.di.modules.AndroidModule;
import com.bundle.bundle_rave_pay.di.modules.NetworkModule;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;

import io.flutter.plugin.common.MethodChannel;

import static com.bundle.bundle_rave_pay.RaveConstants.ENCRYPTION_KEY;
import static com.bundle.bundle_rave_pay.RaveConstants.LIVE_URL;
import static com.bundle.bundle_rave_pay.RaveConstants.PAYMENT_TYPE_ACCOUNT;
import static com.bundle.bundle_rave_pay.RaveConstants.PAYMENT_TYPE_BANK_TRANSFER;
import static com.bundle.bundle_rave_pay.RaveConstants.PUBLIC_KEY;
import static com.bundle.bundle_rave_pay.RaveConstants.RAVE_PARAMS;
import static com.bundle.bundle_rave_pay.RaveConstants.STAGING_URL;

public class RavePayActivity extends AppCompatActivity {

    public static String BASE_URL;
    RavePayInitializer ravePayInitializer;

    public static int RESULT_SUCCESS = 111;
    public static int RESULT_PAYMENT_SUCCESS = 203;
    public static int RESULT_ERROR = 222;
    public static int RESULT_CANCELLED = 333;
    public static int RESULT_PAYMENT_PENDING = 444;

    int theme;
    AppComponent appComponent;

    Double amount = 0.0;
    String cvv = "";
    String cardExpiry = "";
    String cardNumber = "";
    boolean saveCard = true;
    String country = "";
    String currencyCode = "";
    String email = "";
    String fName = "";
    String lName = "";
    String narration = "";
    String txRef = "";
    boolean isLive = false;
    String publicKey = PUBLIC_KEY;
    String encryptionKey = ENCRYPTION_KEY;

    boolean isCard = false;
    boolean isAccount = false;
    boolean isTransfer = false;

    BundleCard bundleCard;
    static MethodChannel.Result flutterChannelResult;

    public static Activity activity = null;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(com.bundle.bundle_rave_pay.R.layout.activity_rave_pay);

        BundleData bundleData;

        activity = this;

        bundleData = getIntent().getParcelableExtra(RAVE_PARAMS);

        startCardPayment(bundleData);

        bundleCard = new BundleCard(amount, cvv, cardExpiry, cardNumber, saveCard);

        buildGraph();
        theme = ravePayInitializer.getTheme();

        if (theme != 0) {
            try {
                setTheme(theme);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        setupMainContent();

    }

    private void startCardPayment(BundleData cardBundle) {
        this.amount = cardBundle.getAmount();
        this.cvv = cardBundle.getCvv();
        this.cardExpiry = cardBundle.getCardExpiry();
        this.cardNumber = cardBundle.getCardNumber();
        this.saveCard = cardBundle.isSaveCard();
        this.country = cardBundle.getCountryCode();
        this.currencyCode = cardBundle.getCurrencyCode();
        this.email = cardBundle.getEmail();
        this.fName = cardBundle.getFirstName();
        this.lName = cardBundle.getLastName();
        this.txRef = cardBundle.getTransactionReference();
        this.isLive = cardBundle.isLive();
        this.publicKey = cardBundle.getPublicKey();
        this.encryptionKey = cardBundle.getEncryptionKey();
        this.narration = cardBundle.getNarration();

        this.isCard = cardBundle.isCard();
        this.isAccount = cardBundle.isAccount();
        this.isTransfer = cardBundle.isBankTransfer();

        ravePayInitializer = setupManager();
    }

    private RavePayInitializer setupManager() {
        RavePayManager ravePayManager = new RavePayManager(this).setAmount(amount)
                .setPublicKey(publicKey)
                .setEncryptionKey(encryptionKey)
                .setCountry(country)
                .setCurrency(currencyCode)
                .setEmail(email)
                .setfName(fName)
                .setlName(lName)
                .setNarration(narration)
                .setTxRef(txRef)
                .acceptAccountPayments(isAccount)
                .acceptCardPayments(isCard)
                .acceptBankTransferPayments(isTransfer)
                .onStagingEnv(!isLive)
                .shouldDisplayFee(false);

        return ravePayManager.createRavePayInitializer();
    }

    private void setupMainContent() {
        int paymentMethod = ravePayInitializer.getOrderedPaymentTypesList().get(0);

        launchFragment(paymentMethod, bundleCard);
    }

    private void launchFragment(int paymentType, Parcelable data) {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();

        switch (paymentType) {
            case PAYMENT_TYPE_ACCOUNT:
                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, new AccountFragment());
                break;
            case PAYMENT_TYPE_BANK_TRANSFER:
                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, new BankTransferFragment());
                break;
            case RaveConstants.PAYMENT_TYPE_CARD:
                Bundle bundle = new Bundle();

                bundle.putParcelable("card", bundleCard);
                CardFragment cardFragment = new CardFragment();
                cardFragment.setArguments(bundle);

                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, cardFragment);
                break;
            default:
                Log.d("Adding Payment Fragment", "Payment type does not exist in payment types list");
                return;
        }

        try {
            transaction.commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    static void setFlutterResult(MethodChannel.Result methodResult) {
        flutterChannelResult = methodResult;
    }

    static HashMap<String, String> resultMap = new HashMap<>();

    public static HashMap<String, String> jsonToMap(String t) throws JSONException {

        HashMap<String, String> map = new HashMap<>();
        JSONObject jObject = new JSONObject(t);
        Iterator<?> keys = jObject.keys();

        while (keys.hasNext()) {
            String key = (String) keys.next();
            String value = jObject.getString(key);
            map.put(key, value);
        }
        return map;
    }

    static void returnSuccess() {
        resultMap.put("status", "success");
        flutterChannelResult.success(resultMap);
    }

    public static void sendResultToFlutter(int resultCode, String responseAsJson) {
        try {
            if (resultCode == RESULT_PAYMENT_SUCCESS) {
                returnSuccess();
            } else if (resultCode == RESULT_ERROR) {
                resultMap.put("status", "failure");
                flutterChannelResult.success(resultMap);
            } else if (resultCode == RESULT_PAYMENT_PENDING) {
                resultMap.put("status", "pending");
                flutterChannelResult.success(resultMap);
            } else if (resultCode == RESULT_CANCELLED) {
                resultMap.put("status", "cancelled");
                flutterChannelResult.success(resultMap);
            }
            resultMap.clear();
            activity.finish();
        } catch (IllegalStateException e) {
            e.printStackTrace();
        } catch (RuntimeException ignored) {

        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        try {
            resultMap.put("status", "cancelled");
            flutterChannelResult.success(resultMap);
        } catch (IllegalStateException ignored) {

        } catch (RuntimeException ignored) {

        }
    }

    private void buildGraph() {

        if (ravePayInitializer.isStaging()) {
            BASE_URL = STAGING_URL;
        } else {
            BASE_URL = LIVE_URL;
        }

        appComponent = DaggerAppComponent.builder()
                .androidModule(new AndroidModule(this))
                .networkModule(new NetworkModule(BASE_URL))
                .build();
    }

    @Override
    public void onBackPressed() {
        sendResultToFlutter(RESULT_CANCELLED, "");
        super.onBackPressed();
    }

    public AppComponent getAppComponent() {
        return appComponent;
    }

    public RavePayInitializer getRavePayInitializer() {
        return ravePayInitializer;
    }

}