//
//  RaveBankTransferClient.swift
//  bundle_rave_pay
//
//  Created by Johnson Ejezie on 05/04/2020.
//

import Foundation

class RaveBankTransferClient {
    
    static func getBankDetails(amount: String, completion: @escaping (BankTransferDetail?, String?) -> Void) {
        var param:[String:Any] = [
            "PBFPubKey": RaveConfig.sharedConfig().publicKey!,
            "amount": amount,
            "email": RaveConfig.sharedConfig().email!,
            "payment_type":"banktransfer",
            "currency": RaveConfig.sharedConfig().currencyCode,
            "country": RaveConfig.sharedConfig().country,
            "IP": getIFAddresses().first!,
            "txRef": RaveConfig.sharedConfig().transcationRef!,
            "is_bank_transfer": true,
            "device_fingerprint": (UIDevice.current.identifierForVendor?.uuidString)!
        ]

        if let meta = RaveConfig.sharedConfig().meta {
            param.merge(["meta":meta])
        }
        
        if let narrate = RaveConfig.sharedConfig().narration{
            param.merge(["narration":narrate])
        }
        let jsonString  = param.jsonStringify()
        let secret = RaveConfig.sharedConfig().encryptionKey!
        let data =  TripleDES.encrypt(string: jsonString, key:secret)
        let base64String = data?.base64EncodedString()
        
        
        let reqbody = [
            "PBFPubKey": RaveConfig.sharedConfig().publicKey!,
            "client": base64String!, // Encrypted $data payload here.
            "alg": "3DES-24"
        ]

        RavePayService.getBankTransferDetails(reqbody, resultCallback: { res in
            if let status = res?["status"] as? String {
                if status == "success"{
                    if let result = res?["data"] as? Dictionary<String,AnyObject>,
                        let chargeResponse = result["response_code"] as? String {
                        if chargeResponse == "02" {
                            let bankDetails = BankTransferDetail.init(json: result)
                            DispatchQueue.main.async {
                               completion(bankDetails, nil)
                            }
                        } else {
                            completion(nil, "Opps failed to get bank details. Try later")
                        }
                    } else {
                        completion(nil, "Opps failed to get bank details. Try later")
                    }
                } else {
                    completion(nil, "Opps failed to get bank details. Try later")
                }
            }
        }) { (err) in
           completion(nil, "Opps failed to get bank details. Try later")
        }
    }
}
