//
//  RavePayService.swift
//  RaveMobile
//
//  Created by Olusegun Solaja on 19/07/2017.
//  Copyright © 2017 Olusegun Solaja. All rights reserved.
//

import UIKit
import Alamofire

class RavePayService: NSObject {
    
    class func queryTransaction(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        AF.request(RaveURLHelper.getV2URL("QUERY_TRANSACTION",withURLParam: [:]),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
    }
        
    class func mpesaQueryTransaction(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("MPESA_QUERY_TRANSACTION",withURLParam: [:]),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
    }
    
    class func getSavedCards(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:SavedCardResponse) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("SAVE_CARD_LOOKUP"),method: .post, parameters: bodyParam).responseData {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success:
                let decoder = JSONDecoder()
               if let data = res.data{
                   if let res = try? decoder.decode(SavedCardResponse.self, from:data){
                       resultCallback(res)
                   }else{
                       errorCallback("An Error has occured please try again")
                   }
               }
            }
        }
        
        
    }
    class func getFee(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
       // print(bodyParam.description)
        AF.request(RaveURLHelper.getURL("FEE"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
        
        
    }
    class func removeSavedCard(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("REMOVE_SAVEDCARD"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
        
        
    }
    
    class func sendOTP(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ result:String) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("SEND_SAVEDCARD_OTP"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
               let result = value as? Dictionary<String,AnyObject>
               if let status = result?["status"] as? String{
                   if status == "success"{
                    resultCallback("Enter the OTP sent to your mobile phone and email address to continue")
                   }
               }else{
                   let message = result?["message"] as? String
                   errorCallback(message ?? "An error occured")
               }
            }
        }
        
        
    }
    class func getBanks(resultCallback:@escaping (_ result:[Bank]?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("BANK_LIST"),method: .get, parameters: nil).responseData {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success:
                let decoder = JSONDecoder()
                if let data = res.data{
                    if let res = try? decoder.decode([Bank].self, from:data){
                            resultCallback(res)
                    }else{
                        errorCallback("An Error has occured please try again")
                    }
                }
            }
        }
        
    }
    class func charge(_ bodyParam:Dictionary<String,Any>,resultCallback:@escaping (_ Result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("CHARGE_CARD"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
                switch res.result {
                case .failure(let error):
                    errorCallback( error.localizedDescription)
                case .success(let value):
                    let result = value as! Dictionary<String,AnyObject>
                    resultCallback(result)
                }
            }
        
        
     }
    class func chargeWithToken(_ bodyParam:Dictionary<String,Any>,resultCallback:@escaping (_ Result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("CHARGE_WITH_TOKEN"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
        
        
    }
    
    class func validateCardOTP(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ Result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("VALIDATE_CARD_OTP"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
        
    }
    class func validateAccountOTP(_ bodyParam:Dictionary<String,String>,resultCallback:@escaping (_ Result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
        
        AF.request(RaveURLHelper.getURL("VALIDATE_ACCOUNT_OTP"),method: .post, parameters: bodyParam).responseJSON {
            (res) -> Void in
            switch res.result {
            case .failure(let error):
                errorCallback( error.localizedDescription)
            case .success(let value):
                let result = value as! Dictionary<String,AnyObject>
                resultCallback(result)
            }
        }
    }
    
    class func getBankTransferDetails(_ bodyParam:Dictionary<String,Any>,resultCallback:@escaping (_ Result:Dictionary<String,AnyObject>?) -> Void ,errorCallback:@escaping (_ err:String) -> Void ){
           let manager = Alamofire.Session.default
           manager.session.configuration.timeoutIntervalForRequest = 30
           manager.session.configuration.timeoutIntervalForResource = 30
        let baseURL = RaveConfig.sharedConfig().isStaging ? RaveConstants.baseURL() : RaveConstants.liveBaseURL()
           manager.request(baseURL+"/flwv3-pug/getpaidx/api/charge", method: .post, parameters: bodyParam).responseJSON {
               (res) -> Void in
               switch res.result {
               case .success(let value):
                   let result = value as? Dictionary<String,AnyObject>
                   resultCallback(result)
               case .failure(let e):
                   errorCallback( e.localizedDescription)
               }
           }
       }
}
