//
//  BankTransferView.swift
//  bundle_rave_pay
//
//  Created by Johnson Ejezie on 05/04/2020.
//

import UIKit

class BankTransferView: UIView {
        
    lazy var titleInfo: UILabel = {
        let label = UILabel()
        label.text = "Please make a bank transfer to Bundle Wallet Technology Ltd"
        label.translatesAutoresizingMaskIntoConstraints = false
        label.textAlignment = .center
        label.numberOfLines = 0
        label.font = UIFont.systemFont(ofSize: 17, weight: .regular)
        return label
    }()
    
    lazy var payButton: UIButton = {
        let button = UIButton(type: .system)
        button.setTitle("I have made the transfer", for: .normal)
        button.setTitleColor(.white, for: .normal)
        button.backgroundColor = UIColor(hex: "#210A54")
        button.layer.cornerRadius = 4
        button.translatesAutoresizingMaskIntoConstraints = false
        return button
    }()
    
    lazy var fieldContainer: UIView = {
        let view = UIView()
        view.backgroundColor = .clear
        view.layer.cornerRadius = 6
        view.clipsToBounds = true
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    private func createKeyLabel(text: String) -> UILabel {
        let label = UILabel()
        label.text = text
        label.numberOfLines = 0
        label.translatesAutoresizingMaskIntoConstraints = false
        label.textColor = UIColor.black
        label.font = UIFont.systemFont(ofSize: 15, weight: .regular)
        return label
    }
    
    private func createValueLabel(text: String) -> UILabel {
        let label = UILabel()
        label.text = text
        label.numberOfLines = 0
        label.textAlignment = .right
        label.translatesAutoresizingMaskIntoConstraints = false
        label.textColor = UIColor.black
        label.font = UIFont.systemFont(ofSize: 15, weight: .regular)
        return label
    }
    
    private func createStackView(key: UILabel, value: UILabel) -> UIStackView {
        let stackView = UIStackView(arrangedSubviews: [
            key,
            value
        ])
        stackView.axis = .horizontal
        stackView.distribution = .fillEqually
        stackView.spacing = 8
        return stackView
    }
    
    lazy var warningNoteContainer: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor(red: 251/255.0, green: 233/255.0, blue: 232/255.0, alpha: 1)
        view.layer.cornerRadius = 5
        view.clipsToBounds = true
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    lazy var warningNoteLabel: UILabel = {
        let fullText = "NOTE: DO NOT save as beneficiary, as the bank details might vary next time"
        let boldText = "NOTE: DO NOT"
        let label = UILabel()
        label.translatesAutoresizingMaskIntoConstraints = false
        label.textAlignment = .center
        label.numberOfLines = 0
        label.textColor = UIColor(red: 219/255.0, green: 84/255.0, blue: 85/255.0, alpha: 1)
        let font = UIFont.systemFont(ofSize: 17, weight: .regular)
        let attributedString = NSMutableAttributedString(string: fullText, attributes: [NSAttributedString.Key.font : font])
        let boldFontAttribute: [NSAttributedString.Key: Any] = [NSAttributedString.Key.font : UIFont.systemFont(ofSize: 17, weight: .bold)]
        let range = (fullText as NSString).range(of: boldText)
        attributedString.addAttributes(boldFontAttribute, range: range)
        label.attributedText = attributedString
        return label
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        backgroundColor = UIColor(hex: "#F2F2F2")
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func setBankDetails(bankDetails: BankTransferDetail) {
        setupConstriant(bankDetails: bankDetails)
    }
    
    private func setupConstriant(bankDetails: BankTransferDetail) {
        addSubview(titleInfo)
        addSubview(fieldContainer)
        addSubview(payButton)
        warningNoteContainer.addSubview(warningNoteLabel)
        addSubview(warningNoteContainer)
        let stackView = UIStackView(arrangedSubviews: [
            createStackView(key: createKeyLabel(text: "Amount"), value: createValueLabel(text: "\(String(bankDetails.amount!).toCountryCurrency(code: RaveConfig.sharedConfig().currencyCode))")),
            createStackView(key: createKeyLabel(text: "Account Number"), value: createValueLabel(text: bankDetails.accountnumber ?? "")),
            createStackView(key: createKeyLabel(text: "Bank Name"), value: createValueLabel(text: bankDetails.bankname ?? "")),
            createStackView(key: createKeyLabel(text: "Beneficiary Name"), value: createValueLabel(text: bankDetails.beneficiary))
        ])
        stackView.axis = .vertical
        stackView.distribution = .fillProportionally
        stackView.spacing = 20
        fieldContainer.addSubview(stackView)
        stackView.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
            stackView.topAnchor.constraint(equalTo: fieldContainer.topAnchor),
            stackView.bottomAnchor.constraint(equalTo: fieldContainer.bottomAnchor),
            stackView.leadingAnchor.constraint(equalTo: fieldContainer.leadingAnchor, constant: 8),
            stackView.trailingAnchor.constraint(equalTo: fieldContainer.trailingAnchor, constant: -8),
            titleInfo.leadingAnchor.constraint(equalTo: leadingAnchor, constant:20),
            titleInfo.topAnchor.constraint(equalTo: topAnchor, constant:20),
            titleInfo.trailingAnchor.constraint(equalTo: trailingAnchor, constant:-20),
            titleInfo.heightAnchor.constraint(equalToConstant: 85),
            
            fieldContainer.topAnchor.constraint(equalTo: titleInfo.bottomAnchor, constant: 24),
            fieldContainer.leadingAnchor.constraint(equalTo: titleInfo.leadingAnchor),
            fieldContainer.trailingAnchor.constraint(equalTo: titleInfo.trailingAnchor),
            
            payButton.centerXAnchor.constraint(equalTo: fieldContainer.centerXAnchor),
            payButton.topAnchor.constraint(equalTo: fieldContainer.bottomAnchor, constant: 40),
            payButton.heightAnchor.constraint(equalToConstant: 50),
            payButton.widthAnchor.constraint(equalToConstant: 250),
            
            warningNoteLabel.topAnchor.constraint(equalTo: warningNoteContainer.topAnchor, constant: 20),
            warningNoteLabel.bottomAnchor.constraint(equalTo: warningNoteContainer.bottomAnchor, constant: -20),
            warningNoteLabel.leadingAnchor.constraint(equalTo: warningNoteContainer.leadingAnchor, constant: 20),
            warningNoteLabel.trailingAnchor.constraint(equalTo: warningNoteContainer.trailingAnchor, constant: -20),
            
            warningNoteContainer.topAnchor.constraint(equalTo: payButton.bottomAnchor, constant: 40),
            warningNoteContainer.leadingAnchor.constraint(equalTo: self.leadingAnchor, constant: 20),
            warningNoteContainer.trailingAnchor.constraint(equalTo: self.trailingAnchor, constant: -20),
        ])
    }
}
