import Flutter
import UIKit

public class SwiftBundleRavePayPlugin: NSObject, FlutterPlugin {
    var bundlePayFlutterResult: FlutterResult!
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "bundle_rave_pay", binaryMessenger: registrar.messenger())
    let instance = SwiftBundleRavePayPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    
    if call.method == "initializePayment" {
        self.bundlePayFlutterResult = result
        processPayment(call)
    }
  }
    
    private func processPayment(_ call: FlutterMethodCall) {
        guard let dictionary = (call.arguments as! [String: Any])["body"] as? [String: Any] else {
            bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Missing some required data", details: nil))
            return
        }
        guard let publicKey = dictionary["publicKey"],
        let encryptionKey = dictionary["encryptionKey"],
        let currency = dictionary["currencyCode"],
        let transactionRef = dictionary["transactionReference"],
        let email = dictionary["email"],
        let paymentMethod = dictionary["paymentMethod"],
            let narration = dictionary["narration"],
        let isLive = dictionary["isLive"] else {
            bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Missing some required data", details: nil))
            return
        }
        let config = RaveConfig.sharedConfig()
        if ("\(paymentMethod)" == "account") {
            config.selectedMethod = .BANK_ACCOUNT
        } else if ("\(paymentMethod)" == "card") {
            guard let cardExpiry = dictionary["cardExpiry"] as? String,
            let cardNumber = dictionary["cardNumber"] as? String,
            let cvv = dictionary["cvv"] as? String else {
                bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Missing some required data", details: nil))
                return
            }
            config.selectedMethod = .CARD(cardNumber, cardExpiry, cvv)
        } else if ("\(paymentMethod)" == "transfer") {
            config.selectedMethod = .BANK_TRANSFER
        } else {
            bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Unknow payment method", details: nil))
        }
        
        config.email = "\(email)"
        config.transcationRef = "\(transactionRef)"
        config.country = "\(dictionary["country"]!)"
        config.currencyCode = "\(currency)"
        config.publicKey = "\(publicKey)"
        config.encryptionKey = "\(encryptionKey)"
        config.isStaging = "\(isLive)" == "false" ? true : false
        config.firstName = dictionary["firstName"] as? String ?? ""
        config.lastName = dictionary["lastName"] as? String ?? ""
        config.narration = narration as? String ?? ""
        let controller = UIApplication.shared.delegate!.window!?.topViewController()
        let raveController = NewRavePayViewController()
        let nav = UINavigationController(rootViewController: raveController)
        raveController.amount = "\(dictionary["amount"]!)"
        raveController.delegate = self
        nav.modalPresentationStyle = .fullScreen
        controller?.present(nav, animated: true)
    }
}

extension SwiftBundleRavePayPlugin: RavePayProtocol {
    public func tranasctionSuccessful(flwRef: String?, responseData: [String : Any]?) {
        bundlePayFlutterResult(responseData)
    }

    public func tranasctionFailed(flwRef: String?, responseData: [String : Any]?) {
        bundlePayFlutterResult(responseData)
    }
}

extension UIWindow {
    func topViewController() -> UIViewController? {
        var top = self.rootViewController
        while true {
            if let presented = top?.presentingViewController {
                top = presented
            } else if let nav = top as? UINavigationController {
                top = nav.visibleViewController
            } else if let tab = top as? UITabBarController {
                top = tab.selectedViewController
            } else {
                break
            }
        }
        return top
    }
}
